function [ shape_space_model ] = train_diffeomorphic_model(param, numimgs)
    %Main wrapper for Taraz's LDDMM shapespace training function
    %default parameters take from test_shape_space_construction_synthesis
    %

% Aug 30, 2013 G. Johnson    Changed they way files are input into the
%                            diffeomorphic model function
     
    if ~isfield(param.model, 'diffeomorphic') 
        param.model.diffeomorphic = true;
    end
    
    param.model.diffeomorphic = ml_initparam(param.model.diffeomorphic, ...
        struct( 'use_distance_matrix_completion', true, ...
                'minimum_relative_semidiameter', 1 / 4, ...
                'maximum_relative_semidiameter', 2 / 3, ...
                'generate_cycle', true, ...
                'useCurrentResults', false, ...
                'tempdir', [param.tempparent filesep 'diffeomorphic']));

    if ~exist(param.model.diffeomorphic.tempdir, 'dir')
        mkdir(param.model.diffeomorphic.tempdir)
    end
            
    options = struct();
    options.save_location = [param.model.diffeomorphic.tempdir];
    
    [imfunc, min_im_size] = get_diffeo_image_function(param);
    image_width = min(min_im_size(1:2));
    
    options.number_images = numimgs;
    options.image_function = imfunc;
    
    options.voxel_size = param.model.resolution;
    options.shape_aspect_ratio = [1, 1, 1];
    
    %use filter that is 1/10 the size of the image
    options.filter_radius = ceil(min_im_size(1)/10);
    options.window_radius = min_im_size(1);
    
    %use z_radius kernel that is 1/5 the size of the z dimension
    options.kernel_z_radius = ceil(min_im_size(3)/5);
    options.convergence_absolute_error = (mean([param.model.diffeomorphic.minimum_relative_semidiameter, param.model.diffeomorphic.maximum_relative_semidiameter]) * image_width * 2 * pi) / 10;

    options.maximum_deformation_per_step = ones(1, 3) * 1; 
%     options.maximum_deformation_per_step = ones(1, 3) * .25; 
    
    options.convergence_absolute_error_difference = 0.5;
    
    options.useCurrentResults = param.model.diffeomorphic.useCurrentResults;
    
    if param.model.diffeomorphic.use_distance_matrix_completion
        options.desired_shape_space_dimensionality = 2;
        % shape_space_options.desired_shape_space_dimensionality = 5;
    end
    
    
    shape_space_model = train_shape_space_model(options);
    
    shape_space_model.imfunc = imfunc;
    shape_space_model.numimgs = numimgs;
    shape_space_model.imsize = min_im_size;
    shape_space_model.name = 'diffeomorphic model of the cell';
    shape_space_model.type = 'diffeomorphic';
    
    shape_space_model.version = 1.0;

end


