function FV = getMeshPoints(image)
%GETMESHPOINTS This function gets mesh points given an image
%
%Inputs: 
%image = a 2D or 3D image 
%
%Outputs:
%FV = a struct containing the vertices etc for the mesh
%

%Author: Devin Sullivan September 19,2013
% Copyright (C) 2012 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.

%Smooth the image a little, should probably not hard code 11.
    D = smooth3(squeeze(image),'gaussian',[11,11,11]);
    
    [x,y,z] = meshgrid(1:size(image,2),1:size(image,1),1:size(image,3));
    x = x-mean(x(:));
    y = y-mean(y(:));
    z = z-mean(z(:));
    v = D;
    
    %make iso-surface (Mesh) of skin
    %should not hard code this.
    patchsample = 0.05;
    
    FV = isosurface(D,0.5);
    % FV.vertices = FV.vertices-repmat(shiftvector_flag,size(FV.vertices,1),1);
    FV = reducepatch(FV,patchsample);